#!/usr/bin/env python3
import os

from .loader import Loader
from .utils import CUSTOM_CATEGORIES, DATASET_DIR


class CustomCategory:
    """
    The CustomCategory class represents a custom category in the Doda dataset.

    It provides methods to access and retrieve information from the custom category, such as the number of entries,
    available columns, retrieving a specific column, retrieving a specific row, and getting a value from a row.
    """
    def __init__(self, type, category) -> None:
        self.__dataset = DATASET_DIR
        if type not in ['semantic', 'syntactic', 'x-tra', 'ongoing']:
            raise ValueError(f'Invalid category type: {type}')
        else:
            self.__type = type
        if type != 'ongoing' and category not in CUSTOM_CATEGORIES:
            raise ValueError(f'Invalid CustomCategory: {category}, try Category class instead')
        type_dir = self.type + ' categories' if self.type in ['semantic', 'syntactic'] else self.type
        if (category + '.csv') not in os.listdir(f'{self.__dataset}/{type_dir}'):
            raise ValueError(f'Invalid {self.type} category: {category}')
        else:
            self.__category = category
        try:
            self.__loader = Loader(self.__dataset, f'{type_dir}/{category}.csv')
        except ValueError:
            raise ValueError('Error loading category file')

    @property
    def type(self) -> str:
        """
        Returns the type of the category.
        
        Returns:
            The type of the category.
        """
        return self.__type

    @property
    def category(self) -> str:
        """
        Returns the name of the category.

        Returns:
            The name of the category.
        """
        return self.__category
    
    def entries(self) -> dict[str, int]:
        """
        Returns the number of entries in the category.
        
        Returns:
            A dictionary containing the number of entries in the category.
        """
        shape = self.__loader.get_shape()
        return {
            'rows': shape[0],
            'columns': shape[1]
        }
    
    def get_all_columns(self) -> list[str]:
        """
        Returns a list of available columns in the category.
        
        Returns:
            A list of available columns in the category.
        """
        return self.__loader.columns

    def get_column(self, column: str) -> list:
        """
        Returns a list of values in the specified column.
        
        Args:
            column: The name of the column to retrieve.
        
        Returns:
            A list of values in the specified column.
        """
        try:
            return self.__loader.get_column(column)
        except ValueError:
            raise ValueError(f'Invalid column: {column}')
    
    def get_row(self, column: str, value: str) -> dict[str, str]:
        """
        Returns a dictionary containing the values of the specified row.

        Args:
            column: The name of the column to search in.
            value: The value to search for.
        
        Returns:
            A dictionary containing the values of the specified row.
        """
        try:
            return self.__loader.get_row(column=column, value=value)
        except ValueError:
            raise ValueError(f'Invalid column: {column}')
    
    def get_column_for_row(self, column: str, value: str, return_column: str) -> str:
        """
        Returns the value of the specified column for the specified row.
        
        Args:
            column: The name of the column to search in.
            value: The value to search for.
            return_column: The name of the column to return the value from.
            
        Returns:
            The value of the specified column for the specified row.
        """
        try:
            return self.__loader.get_word(column=column, value=value, return_column=return_column)
        except ValueError:
            raise ValueError(f'Invalid columns')



#!/usr/bin/env python3
"""
This module contains unittests for the `Category` class.
"""
import unittest
import random
import os
import pandas as pd
import numpy as np

from pydoda import Pydoda, Category
from pydoda.utils import DATASET_DIR, CUSTOM_CATEGORIES


class TestCategory(unittest.TestCase):
    """
    This class contains unittests for the `Category` class.
    """

    maxDiff = None
    categories = Pydoda().classes()['Category']
    types = ['semantic', 'syntactic', 'x-tra']

    def setUp(self):
        """
        This method is run before each test.
        """
        self.type = random.choice(self.types)
        type_categories = Pydoda().all()[self.type]
        category_name = random.choice(self.categories)
        while category_name not in type_categories:
            category_name = random.choice(self.categories)
        self.category = category_name
        self.cat = Category(self.type, self.category)
        self.path = f'{DATASET_DIR}/'
        if self.type in ['semantic', 'syntactic']:
            self.path += f'{self.type} categories/'
        else:
            self.path += f'{self.type}/'
        self.df = pd.read_csv(f'{self.path}{self.category}.csv')
        self.df = self.df.replace(np.nan, None)
        print(f'\nRunning test for {self.type} category: {self.category} in path: {self.path}\n')


    def test_instantiation(self):
        """
        This method tests the instantiation of the `Category` class.
        """
        with self.assertRaises(ValueError):
            Category('invalid type', random.choice(self.categories))
        with self.assertRaises(ValueError):
            Category(random.choice(self.types), 'invalid category')
        with self.assertRaises(ValueError):
            Category(random.choice(self.types), random.choice(CUSTOM_CATEGORIES))
    
    def test_type(self):
        """
        This method tests the `type` property of the `Category` class.
        """
        self.assertEqual(self.cat.type, self.type)
    
    def test_category(self):
        """
        This method tests the `category` property of the `Category` class.
        """
        self.assertEqual(self.cat.category, self.category)
    
    def test_entries(self):
        """
        This method tests the `entries` method of the `Category` class.
        """
        shape = self.cat.entries()
        self.assertIsInstance(shape, dict)
        self.assertEqual(len(shape), 2)
        self.assertIsInstance(shape['rows'], int)
        self.assertIsInstance(shape['columns'], int)
        self.assertGreater(shape['rows'], 0)
        self.assertGreater(shape['columns'], 0)
        rows, columns = self.df.shape
        self.assertEqual(shape['rows'], rows)
        self.assertEqual(shape['columns'], columns)
    
    def test_get_spellings(self):
        """
        This method tests the `get_spellings` method of the `Category` class.
        """
        spellings = self.cat.get_spellings()
        self.assertIsInstance(spellings, list)
        self.assertGreater(len(spellings), 0)
        spellings.append('eng')
        self.assertEqual(spellings, list(self.df.columns))
    
    def test_get_english_words(self):
        """
        This method tests the `get_english_words` method of the `Category` class.
        """
        words = self.cat.get_english_words()
        self.assertIsInstance(words, list)
        self.assertGreater(len(words), 0)
        self.assertEqual(words, self.df['eng'].tolist())
    
    def test_get_darija_words(self):
        """
        This method tests the `get_darija_words` method of the `Category` class.
        """
        spellings = self.cat.get_spellings()
        for spelling in spellings:
            words = self.cat.get_darija_words(spelling)
            self.assertIsInstance(words, list)
            self.assertGreater(len(words), 0)
            self.assertEqual(words, self.df[spelling].tolist())
    
    def test_get_darija_translation(self):
        """
        This method tests the `get_darija_translation` method of the `Category` class.
        """
        word = random.choice(self.df['eng'].tolist())
        translation = self.df[self.df['eng'] == word]['n1'].tolist()[0]
        self.assertEqual(self.cat.get_darija_translation(word), translation)
    
    def test_get_english_translation(self):
        """
        This method tests the `get_english_translation` method of the `Category` class.
        """
        word = random.choice(self.df['n1'].tolist())
        translation = self.df[self.df['n1'] == word]['eng'].tolist()[0]
        self.assertEqual(self.cat.get_english_translation(word), translation)




#!/usr/bin/env python3
import os

from .utils import DATASET_DIR, CUSTOM_CATEGORIES


class Pydoda:
    """
    The Pydoda class provides methods to access and work with Doda datasets.

    This class allows you to retrieve and explore different categories of Doda datasets, such as semantic categories, syntactic categories,
    and additional extra categories. It also provides a method to retrieve all available categories.

    Methods:
        get_semantic_categories(): Returns a list of available semantic categories.
        get_syntactic_categories(): Returns a list of available syntactic categories.
        get_xtra(): Returns a list of available extra categories.
        all(): Returns a dictionary containing all available categories.

    Example usage:
        pydoda = Pydoda()
        semantic_cats = pydoda.get_semantic_categories()
        print(semantic_cats)  # ['category1', 'category2', ...]

    """
    def __init__(self) -> None:
        self.__dataset = DATASET_DIR

    def get_semantic_categories(self) -> list[str]:
        """
        Returns a list of available semantic categories.
        
        Returns:
            A list of available semantic categories.
        """
        return [category.replace('.csv', '') for category in os.listdir(f'{self.__dataset}/semantic categories')]
    
    def get_syntactic_categories(self) -> list[str]:
        """
        Returns a list of available syntactic categories.
        
        Returns:
            A list of available syntactic categories.
        """
        return [category.replace('.csv', '') for category in os.listdir(f'{self.__dataset}/syntactic categories')]
    
    def get_xtra(self) -> list[str]:
        """
        Returns a list of available extra categories.
        
        Returns:
            A list of available extra categories.
        """
        return [category.replace('.csv', '') for category in os.listdir(f'{self.__dataset}/x-tra')]
    
    def get_ongoing(self) -> list[str]:
        """
        Returns a list of ongoing categories.
        
        Returns:
            A list of ongoing categories.
        """
        return [category.replace('.csv', '') for category in os.listdir(f'{self.__dataset}/ongoing')]
    
    def all(self) -> dict[str, list[str]]:
        """
        Returns a dictionary containing all available categories.

        Returns:
            A dictionary containing all available categories.
        """
        return {
            'semantic': self.get_semantic_categories(),
            'syntactic': self.get_syntactic_categories(),
            'x-tra': self.get_xtra(),
            'ongoing': self.get_ongoing(),
            'sentences': ['sentences']
        }

    def classes(self) -> dict[str, list[str]]:
        all = self.get_semantic_categories() + self.get_syntactic_categories() + self.get_xtra()
        categories = [category for category in all if category not in CUSTOM_CATEGORIES]
        return {
            'Category': categories,
            'CustomCategory': CUSTOM_CATEGORIES + self.get_ongoing(),
            'Sentence': ['sentences']
        }



#!/usr/bin/env python3
import os

from .loader import Loader
from .utils import CUSTOM_CATEGORIES, DATASET_DIR


class Category:
    """
    The Category class represents a specific category in the Doda dataset.

    It provides methods to access and retrieve information from the category, such as the number of entries,
    available spellings, English and Darija word translations, and Darija word variations.
    """
    def __init__(self, type, category) -> None:
        self.__dataset = DATASET_DIR
        if type not in ['semantic', 'syntactic', 'x-tra']:
            raise ValueError(f'Invalid category type: {type}')
        else:
            self.__type = type
        if category in CUSTOM_CATEGORIES:
            raise ValueError(f'Invalid Category: {category}, try CustomCategory class instead')
        type_dir = self.type + ' categories' if self.type in ['semantic', 'syntactic'] else self.type
        if (category + '.csv') not in os.listdir(f'{self.__dataset}/{type_dir}'):
            raise ValueError(f'Invalid {self.type} category: {category}')
        else:
            self.__category = category
        try:
            self.__loader = Loader(self.__dataset, f'{type_dir}/{category}.csv')
        except ValueError:
            raise ValueError('Error loading category file')
    
    @property
    def type(self) -> str:
        """
        Returns the type of the category.
        
        Returns:
            The type of the category.
        """
        return self.__type

    @property
    def category(self) -> str:
        """
        Returns the name of the category.

        Returns:
            The name of the category.
        """
        return self.__category
    
    def entries(self) -> dict[str, int]:
        """
        Returns the number of entries in the category.
        
        Returns:
            The number of entries in the category.
        """
        shape = self.__loader.get_shape()
        return {
            'rows': shape[0],
            'columns': shape[1]
        }
    
    def get_spellings(self) -> list[str]:
        """
        Returns a list of available spellings in the category.
        
        Returns:
            A list of available spellings in the category.
        """
        columns = self.__loader.columns
        return [column for column in columns if column != 'eng']
    
    def get_english_words(self) -> list:
        """
        Returns a list of available English words in the category.

        Returns:
            A list of available English words in the category.
        """
        return self.__loader.get_column('eng')

    def get_darija_words(self, spelling: str = 'n1') -> list:
        """
        Returns a list of available Darija words in the category.

        Args:
            spelling: The spelling to retrieve. Defaults to 'n1'.

        Returns:
            A list of available Darija words in the category.
        """
        try:
            return self.__loader.get_column(spelling)
        except ValueError:
            raise ValueError(f'Invalid spelling: {spelling}')
    
    def get_darija_translation(self, word: str, spelling: str = 'n1') -> str:
        """
        Returns the Darija translation of the specified word.

        Args:
            word: The word to retrieve.
            spelling: The spelling to retrieve. Defaults to 'n1'.
        
        Returns:
            The Darija translation of the specified word.
        """
        try:
            return self.__loader.get_word(column='eng', value=word, return_column=spelling)
        except ValueError:
            return None
    
    def get_english_translation(self, word: str, spelling: str = 'n1') -> str:
        """
        Returns the English translation of the specified word.
        
        Args:
            word: The word to retrieve.
            spelling: The spelling to retrieve. Defaults to 'n1'.
            
        Returns:
            The English translation of the specified word.
        """
        try:
            return self.__loader.get_word(column=spelling, value=word, return_column='eng')
        except ValueError:
            return None
    
    def get_darija_variations(self, word: str) -> dict[str, str]:
        """
        Returns a dictionary containing the Darija variations of the specified word.
        
        Args:
            word: The word to retrieve.
        
        Returns:
            A dictionary containing the Darija variations of the specified word.
        """
        try:
            row = self.__loader.get_row(column='eng', value=word)
            if 'eng' in row: del row['eng']
            return row
        except ValueError:
            return None



#!/usr/bin/env python3
import os

from .loader import Loader
from .utils import DATASET_DIR


class Sentence:
    """
    The Sentence class represents a specific sentence in the Doda dataset.

    It provides methods to access and retrieve information from the sentence, such as the number of entries,
    available spellings, English and Darija word translations, and Darija word variations.
    """
    def __init__(self) -> None:
        self.__dataset = DATASET_DIR
        self.__type = self.__category = 'sentences'
        try:
            self.__loader = Loader(self.__dataset, 'sentences/sentences.csv')
        except ValueError:
            raise ValueError('Error loading category file')
    
    @property
    def type(self) -> str:
        """
        Returns the type of the category.
        
        Returns:
            The type of the category.
        """
        return self.__type

    @property
    def category(self) -> str:
        """
        Returns the name of the category.

        Returns:
            The name of the category.
        """
        return self.__category
    
    def entries(self) -> dict[str, int]:
        """
        Returns the number of entries in the category.
        
        Returns:
            The number of entries in the category.
        """
        shape = self.__loader.get_shape()
        return {'rows': shape[0], 'columns': shape[1]}
    
    def get_english_sentences(self) -> list[str]:
        """
        Returns a list of all English sentences in the category.

        Returns:
            A list of all English sentences in the category.
        """
        return self.__loader.get_column('eng')
    
    def get_darija_sentences(self) -> list[str]:
        """
        Returns a list of all Darija sentences in the category.

        Returns:
            A list of all Darija sentences in the category.
        """
        return self.__loader.get_column('darija')
    
    def get_arabic_sentences(self) -> list[str]:
        """
        Returns a list of all arabic-written darija sentences in the category.

        Returns:
            A list of all arabic-written darija sentences in the category.
        """
        return self.__loader.get_column('darija_ar')
    
    def get_darija_translation(self, sentence: str, language: str = 'darija') -> str:
        """
        Returns the Darija translation of the given English sentence.

        Args:
            sentence: The English sentence to translate.
            language: The language of the returned translation. Defaults to 'darija'.

        Returns:
            The Darija translation of the given English sentence.
        """
        try:
            return self.__loader.get_word('eng', sentence, language)
        except ValueError:
            return None
    
    def get_english_translation(self, sentence: str, language: str = 'darija') -> str:
        """
        Returns the English translation of the given Darija sentence.

        Args:
            sentence: The Darija sentence to translate.
            language: The language of the sentence . Defaults to 'darija'.

        Returns:
            The English translation of the given Darija sentence.
        """
        try:
            return self.__loader.get_word(language, sentence, 'eng')
        except ValueError:
            return None

    def get_translation_by_substring(self, substring: str, language: str) -> list[dict[str, str]]:
        """
        Returns a list of translations that contain the given substring.

        Args:
            substring: The substring to search for.
            language: The language of the substring.

        Returns:
            A list of translations that contain the given substring.
        """
        try:
            return self.__loader.filter_rows_by_string(language, substring)
        except KeyError:
            raise ValueError(f'Invalid language: {language}, try "eng" or "darija" or "darija_ar"')
        except ValueError:
            return None



#!/usr/bin/env python3
"""
pydoda: A wrapper python library for the Darija Open Dataset

This module provides the core functionality of the pydoda library, including the main `Pydoda` class,
`Category` class for working with pre-defined categories, and `CustomCategory` class for creating custom
categories.

Classes:
    Pydoda: Main class for dataset info.
    Category: Class for working with pre-defined categories.
    CustomCategory: Class for creating custom categories.

"""
from .py_doda import Pydoda
from .category import Category
from .custom_category import CustomCategory
from .sentence import Sentence


__all__ = ['Pydoda', 'Category', 'CustomCategory']

__version__ = '1.0.0'

def get_version() -> str:
    """Return the version of the pydoda library."""
    return __version__



#!/usr/bin/env python3
"""
This module contains unittests for the `Pydoda` class.
"""
import unittest
import os

from pydoda import Pydoda
from pydoda.utils import CUSTOM_CATEGORIES


class TestPydoda(unittest.TestCase):
    """
    This class contains unittests for the `Pydoda` class.
    """

    maxDiff = None

    def setUp(self):
        """
        This method is run before each test.
        """
        self.pydoda = Pydoda()

    def test_get_semantic_categories(self):
        """
        This method tests the `get_semantic_categories()` method.
        """
        semantic_cats = [cat.replace('.csv', '') for cat in os.listdir('pydoda/dataset/semantic categories')]
        self.assertEqual(self.pydoda.get_semantic_categories(), semantic_cats)
    
    def test_get_syntactic_categories(self):
        """
        This method tests the `get_syntactic_categories()` method.
        """
        syntactic_cats = [cat.replace('.csv', '') for cat in os.listdir('pydoda/dataset/syntactic categories')]
        self.assertEqual(self.pydoda.get_syntactic_categories(), syntactic_cats)
    
    def test_get_xtra(self):
        """
        This method tests the `get_xtra()` method.
        """
        xtra_cats = [cat.replace('.csv', '') for cat in os.listdir('pydoda/dataset/x-tra')]
        self.assertEqual(self.pydoda.get_xtra(), xtra_cats)
    
    def test_all(self):
        """
        This method tests the `all()` method.
        """
        all_cats = {
            'semantic': [cat.replace('.csv', '') for cat in os.listdir('pydoda/dataset/semantic categories')],
            'syntactic': [cat.replace('.csv', '') for cat in os.listdir('pydoda/dataset/syntactic categories')],
            'x-tra': [cat.replace('.csv', '') for cat in os.listdir('pydoda/dataset/x-tra')],
            'sentences': ['sentences']
        }
        self.assertEqual(self.pydoda.all(), all_cats)
    
    def test_classes(self):
        """
        This method tests the `classes()` method.
        """
        all = [cat.replace('.csv', '') for cat in os.listdir('pydoda/dataset/semantic categories')]
        all.extend([cat.replace('.csv', '') for cat in os.listdir('pydoda/dataset/syntactic categories')])
        all.extend([cat.replace('.csv', '') for cat in os.listdir('pydoda/dataset/x-tra')])
        classes = {
            'Category': [cat for cat in all if cat not in CUSTOM_CATEGORIES],
            'CustomCategory': CUSTOM_CATEGORIES,
            'Sentence': ['sentences']
        }
        self.assertEqual(self.pydoda.classes(), classes)



#!/usr/bin/env python3
import os
import pandas as pd
import numpy as np


class Loader:
    """
    The Loader class provides methods to load and work with Doda datasets.
    
    This class allows you to retrieve and explore different categories of Doda datasets, such as
    semantic categories, syntactic categories, and additional extra categories.
    """
    def __init__(self, dataset_dir, category_file_path) -> None:
        self.__dataset = dataset_dir
        self.__category_file = category_file_path
        try:
            self.__df = pd.read_csv(os.path.join(self.__dataset, self.__category_file))
            self.__df = self.__df.replace(np.nan, None)
        except FileNotFoundError:
            raise ValueError('Invalid dataset directory or category file path')
    
    @property
    def columns(self) -> list[str]:
        """
        Returns a list of available columns in the dataset.
        
        Returns:
            A list of available columns in the dataset.
        """
        return self.__df.columns.tolist()
    
    def get_shape(self) -> tuple[int, int]:
        """
        Returns the shape of the dataset.

        Returns:
            A tuple containing the shape of the dataset.
        """
        return self.__df.shape

    def get_column(self, column_name) -> list:
        """
        Returns a list of values in the specified column.
        
        Args:
            column_name: The name of the column to retrieve.
            
        Returns:
            A list of values in the specified column.
        """
        try:
            return self.__df[column_name].tolist()
        except KeyError:
            raise ValueError(f'Invalid column name: {column_name}')
    
    def get_row(self, column, value) -> dict:
        """
        Returns a dictionary containing the values of the specified row.
        
        Args:
            column: The name of the column to search in.
            value: The value to search for.
            
        Returns:
            A dictionary containing the values of the specified row.
        """
        try:
            return self.__df.loc[self.__df[column] == value].to_dict(orient='records')[0]
        except IndexError:
            raise ValueError(f'Invalid value: {value}')
        except KeyError:
            raise ValueError(f'Invalid column name: {column}')
        except ValueError:
            raise ValueError(f'Invalid value: {value}')
    
    def get_word(self, column, value, return_column) -> str:
        """
        Returns the value of the specified column in the specified row.

        Args:
            column: The name of the column to search in.
            value: The value to search for.
            return_column: The name of the column to return the value from.
        
        Returns:
            The value of the specified column in the specified row.
        """
        try:
            return self.__df.loc[self.__df[column] == value, return_column].item()
        except KeyError:
            raise ValueError(f'Invalid columns')
        except ValueError:
            raise ValueError(f'Invalid value: {value}')
    
    def filter_rows_by_string(self, column: str, value: str) -> list[dict[str, str]]:
        """
        Returns a list of dictionaries containing the values of the specified rows.

        Args:
            column: The name of the column to search in.
            value: The value to search for.
        
        Returns:
            A list of dictionaries containing the values of the specified rows.
        """
        try:
            filtered_df = self.__df.loc[self.__df[column].str.contains(value, na=False)]
            filtered_df = filtered_df.infer_objects(copy=False)
            return filtered_df.to_dict(orient='records')
        except KeyError:
            raise ValueError(f'Invalid column name: {column}')
        except ValueError:
            raise ValueError(f'Invalid value: {value}')



#!/usr/bin/env python3
import os


PACKAGE_DIR = os.path.dirname(os.path.abspath(__file__))
DATASET_DIR = os.path.join(PACKAGE_DIR, "dataset")


CUSTOM_CATEGORIES = [
    'malenames',
    'femalenames',
    '(in)definite',
    'conjug_past',
    'conjug_present',
    'imperatives',
    'masculine_feminine_plural',
    'verb_to_noun',
    'idioms',
    'proverbs',
    'shorts',
]



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pydoda",
    version="1.2.0",
    author="Saad Outchakoucht",
    author_email="outsaad03@gmail.com",
    description="A wrapper Python library for working with the DODa dataset",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/saad-out/pydoda",
    packages=setuptools.find_packages(),
    install_requires=[
        "pandas",
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    include_package_data=True,
    package_data={
        "pydoda": [
            "dataset/*",
            "dataset/images/*",
            "dataset/semantic categories/*",
            "dataset/syntactic categories/*",
            "dataset/x-tra/*",
        ]
    },
    python_requires=">=3.6",
)
